package org.gcube.informationsystem.querytemplates.impl.entities;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.gcube.com.fasterxml.jackson.annotation.JsonInclude;
import org.gcube.com.fasterxml.jackson.annotation.JsonInclude.Include;
import org.gcube.com.fasterxml.jackson.annotation.JsonProperty;
import org.gcube.informationsystem.model.reference.properties.Header;
import org.gcube.informationsystem.querytemplates.reference.entities.QueryTemplate;
import org.gcube.informationsystem.querytemplates.reference.properties.TemplateProperty;
import org.gcube.informationsystem.querytemplates.reference.properties.TemplateVariable;
import org.gcube.informationsystem.utils.Version;

/**
 * @author Luca Frosini (ISTI - CNR)
 */
public class QueryTemplatesImpl implements QueryTemplate {

	/**
	 * Generated Serial version UID
	 */
	private static final long serialVersionUID = -1096809036997782113L;
	
	protected Header header;

	protected String name;
	protected String description;
	protected Version version;
	@JsonProperty(value = CHANGELOG_PROPERTY, required = true)
	@JsonInclude(Include.NON_NULL)
	protected Map<Version, String> changelog;
	
	protected TemplateProperty templateProperty;
	protected Set<TemplateVariable> templateDefaultValues;

	public QueryTemplatesImpl() {
		this.changelog = new HashMap<>();
		this.templateDefaultValues = new HashSet<>();
	}
	
	@Override
	public Header getHeader() {
		return header;
	}

	@Override
	public void setHeader(Header header) {
		this.header = header;
	}

	@Override
	public String getName() {
		return name;
	}
	
	@Override
	public void setName(String name) {
		this.name = name;
	}

	@Override
	public String getDescription() {
		return description;
	}

	@Override
	public void setDescription(String description) {
		this.description = description;
	}
	
	@Override
	public Version getVersion() {
		return version;
	}
	
	@Override
	public void setVersion(Version typeVersion) {
		this.version = typeVersion;
	}

	@Override
	public String getVersionAsString() {
		return version.toString();
	}
	
	@Override
	public void setVersion(String version) {
		this.version = new Version(version);
	}

	@Override
	public Map<Version, String> getChangelog() {
		return changelog;
	}

	@Override
	public void addChangelog(Version typeVersion, String changeDescription) {
		if(changelog.get(typeVersion)!=null) {
			throw new RuntimeException("Changelog for version " + typeVersion.toString() + " alread exists");
		}
		changelog.put(typeVersion, changeDescription);
	}
	
	@Override
	public Map<String, String> getChangelogWithVersionAsString() {
		Map<String, String> map = new HashMap<>();
		for (Version typeVersion : changelog.keySet()) {
			map.put(typeVersion.toString(), changelog.get(typeVersion));
		}
		return map;
	}

	@Override
	public void setChangelog(Map<Version, String> changelog) {
		this.changelog = changelog;
	}
	
	@Override
	public void setChangelogWithVersionAsString(Map<String, String> changelog) {
		this.changelog = new HashMap<>();
		for (String version : changelog.keySet()) {
			this.changelog.put(new Version(version), changelog.get(version));
		}
	}
	
	@Override
	public void addChangelog(String version, String changeDescription) {
		Version typeVersion = new Version(version);
		addChangelog(typeVersion, changeDescription);
	}

	@Override
	public TemplateProperty getQueryTemplate() {
		return templateProperty;
	}
	
	@Override
	public void setQueryTemplate(TemplateProperty templateProperty) {
		this.templateProperty = templateProperty;
	}

	@Override
	public Set<TemplateVariable> getQueryTemplateDefaultValues() {
		return templateDefaultValues;
	}
	
	@Override
	public void setQueryTemplateDefaultValues(Set<TemplateVariable> queryTemplateDefaultValues) {
		this.templateDefaultValues = queryTemplateDefaultValues;
	}

	@Override
	public void addQueryTemplateDefaultValues(TemplateVariable queryTemplateDefaultValue) {
		this.templateDefaultValues.add(queryTemplateDefaultValue);
	}

}
